searchData={"items":[{"type":"task","title":"mix oban_web.install","doc":"Installs Oban Web into your Phoenix application\n\nThis task configures your Phoenix application to use the Oban Web dashboard:\n\n* Adds the required `Oban.Web.Router` import\n* Sets up the dashboard route at \"/oban\" within the :dev_routes conditional","ref":"Mix.Tasks.ObanWeb.Install.html"},{"type":"task","title":"Example - mix oban_web.install","doc":"```bash\nmix oban_web.install\n```","ref":"Mix.Tasks.ObanWeb.Install.html#module-example"},{"type":"behaviour","title":"Oban.Web.Resolver","doc":"Web customization is done through a callback module that implements the this behaviour.","ref":"Oban.Web.Resolver.html"},{"type":"behaviour","title":"Usage - Oban.Web.Resolver","doc":"Each callback is optional and resolution falls back to the default implementation when a\ncallback is omittied. Here is an example implementation that defines all callbacks with their\ndefault values for reference:\n\n```elixir\ndefmodule MyApp.Resolver do\n  @behaviour Oban.Web.Resolver\n\n  @impl true\n  def resolve_user(_conn), do: nil\n\n  @impl true\n  def resolve_access(_user), do: :all\n\n  @impl true\n  def resolve_instances(_user), do: :all\n\n  @impl true\n  def resolve_refresh(_user), do: 1\n\n  @impl true\n  def format_job_args(%Oban.Job{args: args}) do\n    inspect(args, charlists: :as_lists, pretty: true)\n  end\n\n  @impl true\n  def format_job_meta(%Oban.Job{meta: meta}) do\n    inspect(meta, charlists: :as_lists, pretty: true)\n  end\n\n  @impl true\n  def format_recorded(recorded, _job) do\n    recorded\n    |> Oban.Web.Resolver.decode_recorded()\n    |> inspect(charlists: :as_lists, pretty: true)\n  end\n\n  @impl true\n  def jobs_query_limit(_state), do: 100_000\n\n  @impl true\n  def hint_query_limit(_qualifier), do: 10_000\n\n  @impl true\n  def bulk_action_limit(_state), do: 1_000\nend\n```\n\nTo use a resolver such as `MyApp.Resolver` defined above, you pass it through as an option to\n`oban_dashboard/2` in your application's router:\n\n```elixir\nscope \"/\" do\n  pipe_through :browser\n\n  oban_dashboard \"/oban\", resolver: MyApp.Resolver\nend\n```","ref":"Oban.Web.Resolver.html#module-usage"},{"type":"behaviour","title":"Overview - Oban.Web.Resolver","doc":"Details about each callback's functionality can be found in the callback docs. Here's a quick\nsummary of each callback and its purpose:\n\n* [Current User](#c:resolve_user/1)—Extract the current user for access controls when the\ndashboard mounts.\n\n* [Action Controls](#c:resolve_access/1)—Restrict which operations users may perform or forbid\naccess to the dashboard.\n\n* [Jobs Query Limit](#c:jobs_query_limit/1)—Control the maximum number of jobs to query when\nfiltering, searching, and otherwise listing jobs.\n\n* [Hint Query Limit](#c:hint_query_limit/1)—Control the maximum number of jobs to search for\nauto-complete hints.\n\n* [Bulk Action Limit](#c:bulk_action_limit/1)—Control the maximum number of jobs that can be\nacted on at once, e.g. cancelled.\n\n* [Format Job Args](#c:format_job_args/1)—Override the default verbose args formatting.\n\n* [Format Job Meta](#c:format_meta_args/1)—Override the default verbose meta formatting.\n\n* [Default Refresh](#c:resolve_refresh/1)—Set the default refresh interval for new sessions.","ref":"Oban.Web.Resolver.html#module-overview"},{"type":"behaviour","title":"Authentication - Oban.Web.Resolver","doc":"By combining `resolver_user/1` and `resolve_access/1` callbacks it's possible to build an\nauthenticaiton solution around the dashboard. For example, this resolver extracts the\n`current_user` from the conn's assigns map and then scopes their access based on role. If it is\na standard user or `nil` then they're redirected to `/login` when the dashboard mounts.\n\n```elixir\ndefmodule MyApp.Resolver do\n  @behaviour Oban.Web.Resolver\n\n  @impl true\n  def resolve_user(conn) do\n    conn.assigns.current_user\n  end\n\n  @impl true\n  def resolve_access(user) do\n    case user do\n      %{admin?: true} -> :all\n      %{staff?: true} -> :read_only\n      _ -> {:forbidden, \"/login\"}\n    end\n  end\nend\n```","ref":"Oban.Web.Resolver.html#module-authentication"},{"type":"type","title":"Oban.Web.Resolver.access/0","doc":"","ref":"Oban.Web.Resolver.html#t:access/0"},{"type":"type","title":"Oban.Web.Resolver.access_option/0","doc":"","ref":"Oban.Web.Resolver.html#t:access_option/0"},{"type":"callback","title":"Oban.Web.Resolver.bulk_action_limit/1","doc":"The maximum number of jobs that can be selected and operated on in bulk.\n\nThe limit may be determined by state, e.g. `:completed` or `:cancelled`, to fine-tune query\nperformance for larger states. Limiting may be disabled with `:infinity`.\n\nWithout a callback implemented all states default to 1000 jobs.","ref":"Oban.Web.Resolver.html#c:bulk_action_limit/1"},{"type":"callback","title":"Example - Oban.Web.Resolver.bulk_action_limit/1","doc":"Bump the limit for all states:\n\n    def bulk_action_limit(_state), do: 5_000\n\nUse a lower limit for `:completed` without any limit for other states:\n\n    def bulk_action_limit(:completed), do: 10_000\n    def bulk_action_limit(_state), do: :infinity","ref":"Oban.Web.Resolver.html#c:bulk_action_limit/1-example"},{"type":"function","title":"Oban.Web.Resolver.decode_recorded/2","doc":"Decode a recorded job's output from a compressed, base64 binary into proper terms.\n\nBy default, decoding uses the `:safe` flag to prevent decoding unsafe data that can be used to\nattack the Erlang runtime. In addition to preventing atom creation, the `:safe` flag will also\nprevent decoding \"executable\" objects such as functions.","ref":"Oban.Web.Resolver.html#decode_recorded/2"},{"type":"function","title":"Example - Oban.Web.Resolver.decode_recorded/2","doc":"Decode a recorded binary:\n\n    iex> Oban.Web.Resolver.decode_recorded(\"g3QAAAABdwRuYW1lbQAAAARvYmFu\")\n    %{name: \"oban\"}\n\nDecode without safety:\n\n    iex> Oban.Web.Resolver.decode_recorded(\"g3QAAAABdwRhdG9tdwd1bmtub3du\", [])\n    %{atom: :unknown}","ref":"Oban.Web.Resolver.html#decode_recorded/2-example"},{"type":"callback","title":"Oban.Web.Resolver.format_job_args/1","doc":"Customize the formatting of job args wherever they are displayed.\n\nThis callback allows for full over args formatting; whether for privacy, brevity, or clarity. By\ndefault, all `args` are preserved in table and detail views.","ref":"Oban.Web.Resolver.html#c:format_job_args/1"},{"type":"callback","title":"Examples - Oban.Web.Resolver.format_job_args/1","doc":"Redact the `\"email\"` field for only the `SecretJob` worker:\n\n    def format_job_args(%Oban.Job{worker: \"MyApp.SecretJob\", args: args}) do\n      args\n      |> Map.replace(\"email\", \"REDACTED\")\n      |> inspect(pretty: true)\n    end\n\n    def format_job_args(job), do: Oban.Web.Resolver.format_job_args(job)","ref":"Oban.Web.Resolver.html#c:format_job_args/1-examples"},{"type":"callback","title":"Oban.Web.Resolver.format_job_meta/1","doc":"Customize the formatting of job meta wherever it is displayed.\n\nThis callback behaves identically to `c:format_job_args/1`, but is used for `meta`.","ref":"Oban.Web.Resolver.html#c:format_job_meta/1"},{"type":"callback","title":"Examples - Oban.Web.Resolver.format_job_meta/1","doc":"Msk the `batch_id` for some secret batch jobs:\n\n    def format_job_meta(%Oban.Job{meta: %{\"batch_id\" => _batch} = meta}) do\n      meta\n      |> Map.replace(\"batch_id\", \"SECRET BATCHES\")\n      |> inspect(pretty: true)\n    end\n\n    def format_job_meta(job), do: Oban.Web.Resolver.format_job_meta(job)","ref":"Oban.Web.Resolver.html#c:format_job_meta/1-examples"},{"type":"callback","title":"Oban.Web.Resolver.format_recorded/2","doc":"Customize the formatting of recorded output wherever it is displayed.\n\nThis callback is similar to `c:format_job_args/1`, but it accepts both the recorded binary and\nthe job to help augment the output.\n\nNote that you **must decode the recorded binary** prior to inspecting it.","ref":"Oban.Web.Resolver.html#c:format_recorded/2"},{"type":"callback","title":"Examples - Oban.Web.Resolver.format_recorded/2","doc":"Disable pretty printing and change the output width to 98 characters:\n\n    def format_recorded(recorded, _job) do\n      recorded\n      |> Oban.Web.Resolver.decode_recorded()\n      |> inspect(pretty: false, width: 98)\n    end\n\nDecode the recorded value without the `:safe` flag set, to allow decoding terms with unknown\natoms:\n\n    def format_recorded(recorded, _job) do\n      recorded\n      |> Oban.Web.Resolver.decode_recorded([])\n      |> inspect(pretty: false, width: 98)\n    end\n\nDisplay job args alongside recorded output:\n\n    def format_recorded(recorded, %Oban.Job{args: args}) when is_map(recorded) do\n      recorded\n      |> Oban.Web.Resolver.decode_recorded()\n      |> Map.put(:args, args)\n      |> inspect(charlists: :as_lists, pretty: true)\n    end","ref":"Oban.Web.Resolver.html#c:format_recorded/2-examples"},{"type":"callback","title":"Oban.Web.Resolver.hint_query_limit/1","doc":"The maximum number of jobs to query for auto-complete suggestions.\n\nThe limit may be determined by qualifier, e.g. `:args` or `:worker`, to fine-tune the impact of\nsuggestion queries. Limiting may be disabled with `:infinity`.\n\nWithout a callback implemented it defaults to 10,000 for all qualifiers.","ref":"Oban.Web.Resolver.html#c:hint_query_limit/1"},{"type":"callback","title":"Example - Oban.Web.Resolver.hint_query_limit/1","doc":"Increase the limit to 20k for all qualifiers:\n\n    def hint_query_limit(_qualifier), do: 20_000\n\nCustomize the limit for `args` and `meta`, falling back to 10k for everything else.\n\n    def hint_query_limit(:args), do: 5_000\n    def hint_query_limit(:meta), do: 2_000\n    def hint_query_limit(_qualifier), do: 10_000","ref":"Oban.Web.Resolver.html#c:hint_query_limit/1-example"},{"type":"callback","title":"Oban.Web.Resolver.jobs_query_limit/1","doc":"The maximum number of jobs to query when displaying jobs.\n\nOrdering is applied _after_ limiting the query. That means the \"oldest\" job displayed will\nalways be within the filter limit. For example, with a limit of 100k jobs and 200k completed\njobs, only the latest 100k are queryable and the oldest 100k are effectively invisible.\n\nIn the interest of speed, limits are only approximate.\n\nThe limit may be determined by state, e.g. `:completed` or `:cancelled`, to fine-tune query\nperformance for larger states. Limiting may be disabled with `:infinity`.\n\nWithout a callback implemented, the `:completed` state defaults to a conservative 100k jobs and all\nother states are `:infinite`.","ref":"Oban.Web.Resolver.html#c:jobs_query_limit/1"},{"type":"callback","title":"Example - Oban.Web.Resolver.jobs_query_limit/1","doc":"Restrict the limit for all states:\n\n    def jobs_query_limit(_state), do: 50_000\n\nUse a conservative the limit for `:completed` without any limit for other states (this is the\ndefault):\n\n    def jobs_query_limit(:completed), do: 100_000\n    def jobs_query_limit(_state), do: :infinity","ref":"Oban.Web.Resolver.html#c:jobs_query_limit/1-example"},{"type":"type","title":"Oban.Web.Resolver.qualifier/0","doc":"","ref":"Oban.Web.Resolver.html#t:qualifier/0"},{"type":"type","title":"Oban.Web.Resolver.refresh/0","doc":"","ref":"Oban.Web.Resolver.html#t:refresh/0"},{"type":"callback","title":"Oban.Web.Resolver.resolve_access/1","doc":"Determine the appropriate access level for a user.\n\nDuring normal operation users can modify running queues and interact with jobs through the\ndashboard. In some situations actions such as pausing a queue may be undesired, or even\ndangerous for operations.\n\nThrough this callback you can tailor precisely which actions the current user can do. The\ndefault access level is `:all`, which permits _all_ users to do any action.\n\nReturning `{:forbidden, path}` prevents loading the dashboard entirely and redirects the user to\nthe provided path.\n\nThe available fine grained access controls are:\n\n* `:pause_queues`\n* `:scale_queues`\n* `:cancel_jobs`\n* `:delete_jobs`\n* `:retry_jobs`\n\nActions which aren't listed are considered disabled.","ref":"Oban.Web.Resolver.html#c:resolve_access/1"},{"type":"callback","title":"Examples - Oban.Web.Resolver.resolve_access/1","doc":"To set the dashboard read only and prevent users from performing any actions at all:\n\n    def resolve_access(_user), do: :read_only\n\nForbid any user that isn't an admin and redirect them to the root:\n\n    def resolve_access(user) do\n      if user.admin?, do: :all, else: {:forbidden, \"/\"}\n    end\n\nAlternatively, you can use the resolved `user` to allow admins write access and keep all other\nusers read only:\n\n    def resolve_access(user) do\n      if user.admin?, do: :all, else: :read_only\n    end\n\nYou can also specify fine grained access for each of the possible dashboard actions.\n\n    def resolve_access(user) do\n      if user.admin? do\n        [cancel_jobs: true, delete_jobs: true, retry_jobs: true]\n      else\n        :read_only\n      end\n    end","ref":"Oban.Web.Resolver.html#c:resolve_access/1-examples"},{"type":"callback","title":"Oban.Web.Resolver.resolve_instances/1","doc":"Explicitly allow Oban instances to be selectable.","ref":"Oban.Web.Resolver.html#c:resolve_instances/1"},{"type":"callback","title":"Examples - Oban.Web.Resolver.resolve_instances/1","doc":"Allow admins to access all instances and others to only access the default:\n\n    def resolve_instances(%{admin?: true}), do: :all\n    def resolve_instances(_user), do: [Oban]","ref":"Oban.Web.Resolver.html#c:resolve_instances/1-examples"},{"type":"callback","title":"Oban.Web.Resolver.resolve_refresh/1","doc":"Determine the initial refresh rate when the dashboard mounts.\n\nThe refresh rate controls how frequently the server pulls statistics from the database, and when\ndata is pushed from the server. The default refresh rate is 1 second.\n\nPossible values are: `1`, `2`, `5`, `15` or `-1` to disable refreshing.\n\nNote that this only sets the default. Users may still choose a different refresh for themselves\nwhile viewing the dashboard and their chosen value will stick for future sessions.","ref":"Oban.Web.Resolver.html#c:resolve_refresh/1"},{"type":"callback","title":"Examples - Oban.Web.Resolver.resolve_refresh/1","doc":"Default to 5 seconds:\n\n    def resolve_refresh(_user), do: 5","ref":"Oban.Web.Resolver.html#c:resolve_refresh/1-examples"},{"type":"callback","title":"Oban.Web.Resolver.resolve_user/1","doc":"Extract the current user from a `Plug.Conn` when the dashboard mounts.\n\nThe extracted user is passed to all of the other callback functions, allowing you to customize the\ndashboard per user or role.\n\nThis callback is expected to return `nil`, a map or a struct. However, the resolved user is only\npassed to other functions in the `Resolver` and as part of the metadata for audit events, so\nyou're free to use any data type you like.","ref":"Oban.Web.Resolver.html#c:resolve_user/1"},{"type":"callback","title":"Examples - Oban.Web.Resolver.resolve_user/1","doc":"Extract the user from the `assigns` map in a typical plug based auth setup:\n\n    def resolve_user(conn) do\n      conn.assigns.current_user\n    end","ref":"Oban.Web.Resolver.html#c:resolve_user/1-examples"},{"type":"type","title":"Oban.Web.Resolver.user/0","doc":"","ref":"Oban.Web.Resolver.html#t:user/0"},{"type":"module","title":"Oban.Web.Router","doc":"Provides mount points for the Web dashboard with customization.","ref":"Oban.Web.Router.html"},{"type":"module","title":"Customizing with a Resolver Callback Module - Oban.Web.Router","doc":"Implementing a `Oban.Web.Resolver` callback module allows you to customize the dashboard\nper-user, i.e. setting access controls or the default refresh rate.\n\nAs a simple example, let's define a module that makes the dashboard read only:\n\n```elixir\ndefmodule MyApp.Resolver do\n  @behaviour Oban.Web.Resolver\n\n  @impl true\n  def resolve_access(_user), do: :read_only\nend\n```\n\nThen specify `MyApp.Resolver` as your resolver:\n\n```elixir\nscope \"/\" do\n  pipe_through :browser\n\n  oban_dashboard \"/oban\", resolver: MyApp.Resolver\nend\n```\n\nSee the `Oban.Web.Resolver` docs for more details.","ref":"Oban.Web.Router.html#module-customizing-with-a-resolver-callback-module"},{"type":"module","title":"Running Multiple Dashboards - Oban.Web.Router","doc":"A single dashboard may connect to any number of running Oban instances. However, it's also\npossible to change the default instance via the `:oban_name` option. For example, given two\nconfigured Oban instances, `Oban` and `MyAdmin.Oban`, you can then mount both dashboards in your\nrouter:\n\n```elixir\nscope \"/\" do\n  pipe_through :browser\n\n  oban_dashboard \"/oban\", oban_name: Oban, as: :oban_dashboard\n  oban_dashboard \"/admin/oban\", oban_name: MyAdmin.Oban, as: :oban_admin_dashboard\nend\n```\n\nNote that the default name is `Oban` or the first found instance, setting `oban_name: Oban` in\nthe example above was purely for demonstration purposes.","ref":"Oban.Web.Router.html#module-running-multiple-dashboards"},{"type":"module","title":"On Mount Hooks - Oban.Web.Router","doc":"You can provide a list of hooks to attach to the dashboard's mount lifecycle. Additional hooks\nare prepended before [Oban Web's own Authentication](Oban.Web.Resolver). For example, to run a\nuser-fetching hook and an activation checking hook before mount:\n\n```elixir\nscope \"/\" do\n  pipe_through :browser\n\n  oban_dashboard \"/oban\", on_mount: [MyApp.UserHook, MyApp.ActivatedHook]\nend\n```","ref":"Oban.Web.Router.html#module-on-mount-hooks"},{"type":"module","title":"Customizing the Socket Connection - Oban.Web.Router","doc":"Applications that use a live socket other than \"/live\" can override the default socket path in\nthe router. For example, if your live socket is hosted at `/oban_live`:\n\n```elixir\nsocket \"/oban_live\", Phoenix.LiveView.Socket\n\nscope \"/\" do\n  pipe_through :browser\n\n  oban_dashboard \"/oban\", socket_path: \"/oban_live\"\nend\n```\n\nIf your application is hosted in an environment that doesn't support websockets you can use\nlongpolling as an alternate transport. To start, make sure that your live socket is configured\nfor longpolling:\n\n```elixir\nsocket \"/live\", Phoenix.LiveView.Socket,\n  longpoll: [connect_info: [session: @session_options], log: false]\n```\n\nThen specify \"longpoll\" as your transport:\n\n```elixir\nscope \"/\" do\n  pipe_through :browser\n\n  oban_dashboard \"/oban\", transport: \"longpoll\"\nend\n```","ref":"Oban.Web.Router.html#module-customizing-the-socket-connection"},{"type":"module","title":"Content Security Policy - Oban.Web.Router","doc":"To secure the dashboard, or comply with an existing CSP within your application, you can specify\nnonce keys for images, scripts and styles.\n\nYou'll configure the CSP nonce assign key in your router, where the dashboard is mounted. For\nexample, to use a single nonce for all three asset types:\n\n```elixir\noban_dashboard(\"/oban\", csp_nonce_assign_key: :my_csp_nonce)\n```\n\nThat instructs the dashboard to extract a generated nonce from the `assigns` map on the plug\nconnection, at the `:my_csp_nonce` key.\n\nInstead, you can specify different keys for each asset type:\n\n```elixir\noban_dashboard(\"/oban\",\n  csp_nonce_assign_key: %{\n    img: :img_csp_nonce,\n    style: :style_csp_nonce,\n    script: :script_csp_nonce\n  }\n)\n```","ref":"Oban.Web.Router.html#module-content-security-policy"},{"type":"macro","title":"Oban.Web.Router.oban_dashboard/2","doc":"Defines an oban dashboard route.\n\nIt requires a path where to mount the dashboard at and allows options to customize routing.","ref":"Oban.Web.Router.html#oban_dashboard/2"},{"type":"macro","title":"Options - Oban.Web.Router.oban_dashboard/2","doc":"* `:as` — override the route name; otherwise defaults to `:oban_dashboard`\n\n* `:on_mount` — declares additional module callbacks to be invoked when the dashboard mounts\n\n* `:oban_name` — name of the Oban instance the dashboard will use for configuration and\n  notifications, defaults to `Oban` or the first instance that can be found.\n\n* `:resolver` — an `Oban.Web.Resolver` implementation used to customize the dashboard's\n  functionality.\n\n* `:socket_path` — a phoenix socket path for live communication, defaults to `\"/live\"`.\n\n* `:transport` — a phoenix socket transport, either `\"websocket\"` or `\"longpoll\"`, defaults to\n  `\"websocket\"`.\n\n* `:csp_nonce_assign_key` — CSP (Content Security Policy) keys used to authenticate image,\n  style, and script assets by pulling a generated nonce out of the connection's `assigns` map. May\n  be `nil`, a single atom, or a map of atoms. Defaults to `nil`.","ref":"Oban.Web.Router.html#oban_dashboard/2-options"},{"type":"macro","title":"Examples - Oban.Web.Router.oban_dashboard/2","doc":"Mount an `oban` dashboard at the path \"/oban\":\n\n    defmodule MyAppWeb.Router do\n      use Phoenix.Router\n\n      import Oban.Web.Router\n\n      scope \"/\", MyAppWeb do\n        pipe_through [:browser]\n\n        oban_dashboard \"/oban\"\n      end\n    end","ref":"Oban.Web.Router.html#oban_dashboard/2-examples"},{"type":"module","title":"Oban.Web.Telemetry","doc":"Oban Web uses `Telemetry` to provide instrumentation and to power logging of dashboard activity.","ref":"Oban.Web.Telemetry.html"},{"type":"module","title":"Action Events - Oban.Web.Telemetry","doc":"An action event is emitted whenever a user performs a write operation with the dashboard, e.g.\npausing a queue, cancelling a job, etc.\n\nThe dashboard emits the following events:\n\n* `[:oban_web, :action, :start]`\n* `[:oban_web, :action, :stop]`\n* `[:oban_web, :action, :exception]`\n\nAction events include the action name, Oban config, the user that performed the action (if\navailable), and relevant metadata. In addition, failed actions provide the error type, the error\nitself, and the stacktrace.\n\nThe following chart shows the _base metadata_ for each event:\n\n| event        | measures       | metadata                                              |\n| ------------ | ---------------| ----------------------------------------------------- |\n| `:start`     | `:system_time` | `:action, :config, :user`                             |\n| `:stop`      | `:duration`    | `:action, :config, :user`                             |\n| `:exception` | `:duration`    | `:action, :config, :user, :kind, :error, :stacktrace` |\n\nFor `:exception` events the metadata includes details about what caused the failure. The `:kind`\nvalue is determined by how an error occurred.\n\nThis chart breaks down the possible actions and their specific metadata:\n\n| action               | metadata           |\n| -------------------- | ------------------ |\n| `:mount`             |                    |\n| `:pause_queue`       | `:queue`           |\n| `:resume_queue`      | `:queue`           |\n| `:pause_all_queues`  |                    |\n| `:resume_all_queues` |                    |\n| `:scale_queue`       | `:queue`, `:limit` |\n| `:cancel_jobs`       | `:job_ids`         |\n| `:delete_jobs`       | `:job_ids`         |\n| `:retry_jobs`        | `:job_ids`         |","ref":"Oban.Web.Telemetry.html#module-action-events"},{"type":"module","title":"Action Logging - Oban.Web.Telemetry","doc":"The `Oban.Web.Telemetry` module ships with a default handler that logs structured JSON for\n`:stop` and `:exception` events. To attach the logger, call `attach_default_logger/1` as your\napplication starts:\n\n```elixir\ndef start(_type, _args) do\n  children = [\n    MyApp.Repo,\n    MyApp.Endpoint,\n    {Oban, oban_opts()}\n  ]\n\n  Oban.Telemetry.attach_default_logger(:info)\n  Oban.Web.Telemetry.attach_default_logger(:info)\n\n  Supervisor.start_link(children, [strategy: :one_for_one, name: MyApp.Supervisor])\nend\n```","ref":"Oban.Web.Telemetry.html#module-action-logging"},{"type":"function","title":"Oban.Web.Telemetry.attach_default_logger/1","doc":"Attaches a structured telemetry handler for logging.","ref":"Oban.Web.Telemetry.html#attach_default_logger/1"},{"type":"function","title":"Options - Oban.Web.Telemetry.attach_default_logger/1","doc":"* `:level` — The log level to use for logging output, defaults to `:info`\n\n* `:encode` — Whether to encode log output as JSON, defaults to `true`","ref":"Oban.Web.Telemetry.html#attach_default_logger/1-options"},{"type":"function","title":"Events - Oban.Web.Telemetry.attach_default_logger/1","doc":"Here is an example of the JSON output for an `action:stop` event:\n\n```json\n{\n  \"action\":\"cancel_jobs\",\n  \"duration\":2544,\n  \"event\":\"action:stop\",\n  \"job_ids\":[290950],\n  \"oban_name\":\"Oban\",\n  \"source\":\"oban_web\",\n  \"user\":1818\n}\n```\n\nEvent metadata is passed through directly along with these constant fields:\n\n* `action` — The reported action, e.g. `cancel_jobs`\n\n* `duration` — Action duration, recorded in native units and logged as microseconds\n\n* `source` — Always \"oban_web\", which is useful for log filtering\n\n* `user` — If the dashboard was mounted with an `Oban.Web.Resolver` that implements\n  `resolve_user/1` this is the user's id, otherwise `null`\n\n* `oban_name` — The instance that the dashboard is linked to, typically this is\n  \"Oban\" unless an application is using multiple Oban instances.","ref":"Oban.Web.Telemetry.html#attach_default_logger/1-events"},{"type":"function","title":"Examples - Oban.Web.Telemetry.attach_default_logger/1","doc":"Attach the logger at the `:debug` log level:\n\n    Oban.Web.Telemetry.attach_default_logger(level: :debug)\n\nDisable JSON encoding output:\n\n    Oban.Web.Telemetry.attach_default_logger(encode: false)","ref":"Oban.Web.Telemetry.html#attach_default_logger/1-examples"},{"type":"extras","title":"Changelog","doc":"# Changelog for Oban Web v2.11\n\nAll notable changes to `Oban.Web` are documented here.","ref":"changelog.html"},{"type":"extras","title":"🥂 Web is Free and Open Source - Changelog","doc":"Oban Web is now [fully open source][fos] and free (as in champagne 😉)! From now on, Oban Web will\nbe [published to Hex][hex] and available for use in your Oban powered applications. See the\nupdated, much slimmer, [installation guide][ins] to get started!\n\nWeb v2.11 is licensed under Apache 2.0, just like Oban and Elixir itself. Previous versions are\ncommercially licensed, therefore private, and won't be published to Hex.\n\nSpecial thanks to all of our customers that supported the project for the past 5 years and made\nopen source Oban, and now Web, possible 💛.\n\n[fos]: https://github.com/oban-bg/oban_web\n[hex]: https://hex.pm/packages/oban_web\n[ins]: installation.md","ref":"changelog.html#web-is-free-and-open-source"},{"type":"extras","title":"🐬🪶 MySQL and SQLite - Changelog","doc":"All official Oban and Oban Pro engines are fully supported. Listing, filtering, ordering, and\nsearching through jobs works for Postgres, MySQL, and SQLite. That includes the particularly\ngnarly issue of dynamically generating and manipulating JSON for filter auto-suggestions! Nested\nargs queries, such as `args.address.city:Edinburgh` work equally well with each engine.","ref":"changelog.html#mysql-and-sqlite"},{"type":"extras","title":"🎛️ Instance Select - Changelog","doc":"The dashboard will now support switching between Oban instances. A new instance select menu in the\nheader allows switching between running Oban instances at runtime. There's no need to mount\nadditional dashboards in your application's router to handle multiple instances. The most recently\nselected instance persists between mounts.\n\n```diff\n- oban_dashboard \"/oban_a\", oban_name: Oban.A\n- oban_dashboard \"/oban_b\", oban_name: Oban.B\n- oban_dashboard \"/oban_c\", oban_name: Oban.C\n+ oban_dashboard \"/oban\"\n```\n\nThis also eliminates the need for additional router configuration, as the dashboard will select\nthe first running Oban instance it finds (with a preference for the default `Oban`).\n\n```diff\n- oban_dashboard \"/oban\", oban_name: MyOban\n+ oban_dashboard \"/oban\"\n```","ref":"changelog.html#instance-select"},{"type":"extras","title":"☯️ Unified Tables - Changelog","doc":"The queue and jobs tables are fully rebuilt with shared, reusable components and matching\nfunctionality. This makes interacting with jobs clearer while queues gain some much requested\nfunctionality:\n\n* Sidebar - a new queue sidebar shows status counts and enables filtering by statuses such as\n  `paused` or `terminating`.\n\n* Filtering - queues are auto-complete filterable just like jobs, making it possible to find\n  queues running on a particular node or narrow down by status.\n\n* Shared Sorting - queue sorting now behaves identically to jobs, through a shared dropdown. \n\n* Uniform Navigation - click on any part of the queue row to navigate to details.\n\n* Condensed Rows  - simplify the queue page by removing nested row components. Extra queue details\n  are in the sub-queue page.","ref":"changelog.html#unified-tables"},{"type":"extras","title":"🕯️ Operate on Full Selection - Changelog","doc":"Apply bulk actions to all selected jobs, not just those visible on the current page.\n\nThis expands the select functionality to extend beyond the current page and include all filtered\njobs, up to a configurable limit. The limit defaults to 1000 and may be overridden with a [resolver\ncallback][rsc].\n\n[rsc]: Oban.Web.Resolver.html#c:bulk_action_limit/1","ref":"changelog.html#operate-on-full-selection"},{"type":"extras","title":"v2.11.2 - 2025-04-21 - Changelog","doc":"","ref":"changelog.html#v2-11-2-2025-04-21"},{"type":"extras","title":"Enhancements - Changelog","doc":"- [Installer] Add igniter powered `oban_web` installer.\n\n  It's now possible to install oban_web with a single igniter command:\n\n  ```bash\n  mix igniter.install oban_web\n  ```\n\n  Or install `oban` and `oban_web` at the same time:\n\n  ```bash\n  mix igniter.install oban,oban_web\n  ```\n\n- [Resolver] Pattern match on `arg` rather than checking for `decorated` annotation.\n\n  Matching on term encoded `arg` is more accurate than checking for decorated metadata. This makes\n  the default `format_job_args` compatible with workflow cascade jobs that don't have any `arg`\n  set.\n\n- [Page] Upgrade bundled assets to use the Phoenix LiveView JavaScript version to v1.10\n\n- [Chart] Replace inline styles with tailwind classes to avoid inline style CSP warnings.","ref":"changelog.html#enhancements"},{"type":"extras","title":"v2.11.1 — 2025-02-06 - Changelog","doc":"","ref":"changelog.html#v2-11-1-2025-02-06"},{"type":"extras","title":"Enhancements - Changelog","doc":"- [Layout] Display Oban.Met version in layout footer\n\n  The Met version is highly relevant to how Web behaves. This also refactors the version display\n  for reuse with consistent conditionals.\n\n- [Layout] Only show Pro version number when available.\n\n  The version footer shows that Pro isn't available rather than showing a `v` followed by a blank\n  space.\n\n- [Jobs] Auto-complete worker priorities from 0 to 9\n\n  Priority completion only matched values from 0..3, but the full range is 0..9.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"- [Jobs] Preserve quotes in `args` and `meta` searches.\n\n  Parsing would strip quotes from args and meta queries. This prevented quoted numeric vaalues to\n  be treated as integers. Now quotes are preserved for `args` and `meta`, just as they are for\n  other qualifiers.\n\n- [Queue Details] Eliminate duplicate id warnings for inputs in queue details.\n\n  The latest live_view alerts when nodes in a view have conflicting ids, which caught a number of\n  instances on the queue details page.\n\n- [Queues] Fix duplicate ids for nodes and queues in sidebar.\n\n- [Search] Correct assigns typo in search component handler.\n\n  The key is `assigns`, not `asigns`.\n\n- [Resolver] Fix resolver access typespec.\n\n  Access options must be a keyword list with boolean values, not just a list of option atoms.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v2.11.0 — 2025-01-16 - Changelog","doc":"","ref":"changelog.html#v2-11-0-2025-01-16"},{"type":"extras","title":"Enhancements - Changelog","doc":"- [Dashboard] Load using non-default Oban instance without any config.\n\n  The dashboard now loads the first running non-default Oban instance, even without anything\n  configured.\n\n- [Jobs] Decode decorated `arg` when formatting job args.\n\n  The decorated `arg` are term encoded and inscrutiable in the standard display. Now the value is\n  decoded and display as the original value.\n\n- [Jobs] Prevent decoding executable functions when displaying `recorded` output.\n\n  By default, recorded content uses the `:safe` flag, which now prevents both atom creation and\n  executable content. This is an extra security precaution to prevent any possible exploits\n  through Web.\n\n- [Queues] Add sidebar similar to the jobs table, including filters to make bulk operations on\n  queues possible.\n\n- [Queues] Add search bar with filtering functionality to query queues.\n\n- [Queues] Add multi-select mode to allow operating on multiple queues at once.\n\n- [Queues] Expose a \"Stop Queues\" operation along with corresponding access controls to prevent\n  unintended shutdown.\n\n- [Resolver] Add `resolve_instances/1` callback to restrict selectable Oban instances.\n\n  Not all instances should be available to all users. This adds the `resolve_instances/1` callback\n  to allow scoping instances by user:\n\n  ```elixir\n  def resolve_instances(%{admin?: true}), do: :all\n  def resolve_instances(_user), do: [Oban]\n  ```\n\n- [Resolver] Add `bulk_action_limit/1` to restrict the number of jobs operated on at once.\n\n  Bulk operations now effect all filtered jobs, not just those visible on the current page. As\n  there may be millions of jobs that match a filter, some limiting factor is necessary. The\n  default of 1000 may be overridden:\n\n  ```elixir\n  def bulk_action_limit(:completed), do: 10_000\n  def bulk_action_limit(_state), do: :infinity\n  ```","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"- [Page] Address deprecations from recent upgrades to Elixir 1.17+, Phoenix 1.5+, and Phoenix\n  LiveView 1.0+\n\n- [Jobs] Always treat args wrapped in quotes as a string when filtering.\n\n  Manually wrapping values in quotes enforces searching by string, rather than as a parsed\n  integer. As values are cast to JSON the type matters, and `123 != \"123\"`.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Overview","doc":"# Overview\n\nOban Web is a view of [Oban's][oba] inner workings that you host directly within your application.\nPowered by [Oban Metrics][met] and [Phoenix Live View][liv], it is distributed, lightweight, and\nfully realtime.\n\n[oba]: https://github.com/oban-bg/oban\n[met]: https://github.com/oban-bg/oban_met\n[liv]: https://github.com/phoenixframework/phoenix_live_view","ref":"overview.html"},{"type":"extras","title":"Features - Overview","doc":"- **🐦‍🔥 Embedded LiveView** - Mount the dashboard directly in your application without any\n  external dependencies.\n\n- **📊 Realtime Charts** - Powered by a custom, distrubted time-series data store that's compacted\n  for hours of efficient storage and filterable by node, queue, state, and worker.\n\n- **🛸 Live Updates** - Monitor background job activity across all queues and nodes in real\n  time, with customizable refresh rates and automatic pausing on blur.\n\n- **🔍 Powerful Filtering** - Intelligently filter jobs by worker, queue, args, tags and more with\n  auto-completed suggestions.\n\n- **🔬 Detailed Inspection** - View job details including when, where and how it was ran (or how\n  it failed to run).\n\n- **🔄 Batch Actions** - Cancel, delete and retry selected jobs or all jobs matching the current\n  filters.\n\n- **🎛️ Queue Controls** - Scale, pause, resume, and stop queues across all running nodes. Queues\n  running with [Oban Pro](https://oban.pro) can also edit global limits, rate limiting, and\n  partitioning.\n\n- **♊ Multiple Dashboards** - Switch between all running Oban instance from a single mount point,\n  or restrict access to some dashboards with exclusion controls.\n\n- **🔒 Access Control** - Allow admins to control queues and interract with jobs while restricting\n  other users to read-only use of the dashboard.\n\n- **🎬 Action Logging** - Use telemetry events to instrument and report all of a user's dashboard\n  activity. A telemetry-powered logger is provided for easy reporting.","ref":"overview.html#features"},{"type":"extras","title":"Installation - Overview","doc":"See the [installation guide](installation.md) for details on installing and configuring Oban Web\nfor your application.","ref":"overview.html#installation"},{"type":"extras","title":"Installation","doc":"# Installation\n\nOban Web is delivered as a hex package named `oban_web`. The package is entirely self contained—it\ndoesn't hook into your asset pipeline at all.\n\nThere are three installation mechanisms available:\n\n- [Semi-Automatic Installation](#semi-automatic-installation) using an igniter powered mix task\n- [Igniter Installation](#igniter-installation) fully automatic installation using igniter\n- [Manual Installation](#manual-installation) add Oban Web and handle all steps manually","ref":"installation.html"},{"type":"extras","title":"Semi-Automatic Installation - Installation","doc":"You can use the `oban_web.install` task without the `igniter.install` escript available.\nFirst, add `oban_web` and `igniter` to your deps in `mix.exs`:\n\n```elixir\n{:oban_web, \"~> 2.11\"},\n{:igniter, \"~> 0.5\", only: [:dev]},\n```\n\nRun `mix deps.get` to fetch `oban_web`, then run the install task:\n\n```bash\nmix oban_web.install\n```\n\nThis will automate all of the manual setup steps for you!","ref":"installation.html#semi-automatic-installation"},{"type":"extras","title":"Igniter Installation - Installation","doc":"For projects that have [igniter][igniter] available, Oban Web can be installed and configured with\na single command:\n\n```bash\nmix igniter.install oban_web\n```\n\nthat will add the latest version of `oban_web` to your dependencies before running the installer,\nthen mount it as `/oban` within the `:dev_routes` conditional.","ref":"installation.html#igniter-installation"},{"type":"extras","title":"Manual Installation - Installation","doc":"Before installing Oban Web, ensure you have:\n\n1. **Oban** - Verify Oban is installed in your application. If not, follow [these\n  instructions][oi] to get started.\n\n2. **Phoenix Live View** - Ensure [Phoenix Live View][plv] is installed and working. If you\n  don't have Live View yet, follow [these instructions][lvi].\n\nAdd `oban_web` to your list of deps in `mix.exs`:\n\n```elixir\n{:oban_web, \"~> 2.11\"}\n```\n\nThen fetch the dependencies:\n\n```bash\nmix deps.get\n```\n\nAfter fetching the package, use the `Oban.Web.Router` to mount the dashboard within your\napplication's `router.ex`:\n\n```elixir\n# lib/my_app_web/router.ex\nuse MyAppWeb, :router\n\nimport Oban.Web.Router\n\n...\n\nscope \"/\" do\n  pipe_through :browser\n\n  oban_dashboard \"/oban\"\nend\n```\n\nHere we're using `\"/oban\"` as the mount point, but it can be anywhere you like. See the\n`Oban.Web.Router` docs for additional options.","ref":"installation.html#manual-installation"},{"type":"extras","title":"Post-Installation - Installation","doc":"After installation (by any method), you should consider the following configuration steps:","ref":"installation.html#post-installation"},{"type":"extras","title":"Secure Dashboard Access - Installation","doc":"After verifying that the dashboard is loading, it's recommended to restrict access to it, either\nwith a [custom resolver's][ac] access controls or [Basic Auth][ba].","ref":"installation.html#secure-dashboard-access"},{"type":"extras","title":"Switch to the PG Notifier - Installation","doc":"PubSub notifications are essential to Web's operation. Oban uses a Postgres based notifier for\nPubSub by default. This notifier is convenient when getting started, but it has a hard 8kb\nrestriction on PubSub messages and busy systems may exceed that occasionally.\n\nTo get the most out of Web's metrics, you should switch to the PG (Process Groups) based notifier\nbuilt on Distributed Erlang.\n\n```diff\n config :my_app, Oban,\n+  notifier: Oban.Notifiers.PG,\n   repo: MyApp.Repo,\n   ...\n```\n\n> #### Clustering Required {: .info}\n>\n> The PG notifier **requires that your app is clustered** together. Otherwise, notifications are\n> local to the current node and **you won't see accurate counts or activity metrics**.","ref":"installation.html#switch-to-the-pg-notifier"},{"type":"extras","title":"Split Web and Worker Nodes - Installation","doc":"If your application runs on separate \"web\" and \"worker\" nodes, you must explicitly include\n`oban_met` as a dependency for the \"worker\" nodes:\n\n```elixir\n{:oban_met, \"~> 1.0\"}\n```","ref":"installation.html#split-web-and-worker-nodes"},{"type":"extras","title":"Customize the Dashboard - Installation","doc":"Web customization is done through the `Oban.Web.Resolver` behaviour. It allows you to enable\naccess controls, control formatting, and provide query limits for filtering and searching. Using a\ncustom resolver is optional, but you should familiarize yourself with the default limits\nand functionality.","ref":"installation.html#customize-the-dashboard"},{"type":"extras","title":"Next Steps - Installation","doc":"Installation is complete and you're all set! Start your Phoenix server, point your browser to\n`/oban` and start monitoring your jobs.\n\n* Configure dashboard behavior with access controls, query limits, and formatting using\n  `Oban.Web.Resolver`\n\n* Attach logging and hook into telemetry events with `Oban.Web.Telemetry`\n\n[igniter]: https://hex.pm/packages/igniter\n[plv]: https://github.com/phoenixframework/phoenix_live_view\n[lvi]: https://github.com/phoenixframework/phoenix_live_view#installation\n[ac]: Oban.Web.Resolver.html#c:resolve_access/1\n[ba]: https://hexdocs.pm/basic_auth/readme.html\n[oi]: installation.html","ref":"installation.html#next-steps"},{"type":"extras","title":"Filtering","doc":"# Filtering\n\nThe filter bar supports advanced syntax to whittle down jobs based on multiple fields. All\nsearches require a qualifier to narrow criteria for fast queries. While the exact-match qualifier\nsyntax is more restrictive than full text searching, it is **vastly faster** in most situations\nand always indexable.\n\n>","ref":"filtering.html"},{"type":"extras","title":"Query Limits {: .info} - Filtering","doc":">\n> To minimize load, only the **most recent 100k jobs** are searched (approximately, across all\n> states). Use the `c:Oban.Web.Resolver.jobs_query_limit/1` callback to configure a different\n> general or per state limit, or disable limiting entirely.","ref":"filtering.html#query-limits-info"},{"type":"extras","title":"Qualifiers - Filtering","doc":"Qualifiers are used to target a specific `Oban.Job` field. The available qualifiers are:\n\n| Qualifier     | Description            | Example                 |\n| ------------- | ---------------------- | ----------------------- |\n| `args.`       | a key or value in args | `args.id:123`           |\n| `meta.`       | a key or value in meta | `meta.batch_id:123`     |\n| `nodes:`      | host name              | `nodes:worker@somehost` |\n| `priorities:` | number from 0 to 9     | `priorities:1`          |\n| `queues:`     | queue name             | `queues:default`        |\n| `tags:`       | tag name               | `tags:super,duper`      |\n| `workers`     | worker module name     | `workers:MyApp.Worker`  |","ref":"filtering.html#qualifiers"},{"type":"extras","title":"Nested Args/Meta Paths - Filtering","doc":"The structured `jsonb` fields, `args` and `meta`, require searching in a path specified with a\n\"dot\" syntax. For example:\n\n* `args.user.plan.name:business`\n* `meta.worker_id:123`\n* `meta.batch_callback_args.some_id:123`\n\nTo aid in path construction and value matching the filter combobox will suggest paths after a `.`\nand values at that path after a `:`.\n\n> #### Multiple Values {: .info}\n> \n> Comma separation doesn't work for `args` or `meta` filters. That's because they use a highly\n> optimized containment query which doesn't support multiple values.","ref":"filtering.html#nested-args-meta-paths"},{"type":"extras","title":"Quoted Searches - Filtering","doc":"If your search query contains whitespace or punctuation you need to surround it with quotation\nmarks. For example:\n\n* `args.address.city:\"New York City\"`\n\nIn practice this isn't much of an issue because only `args` or `meta` may contain whitespace\nanyhow.\n\nYou can't use the following characters as part of your search query:\n\n`, : ; / \\ ' = * ! ? # $ & + ^ | ~ < > ( ) { } [ ]`\n\nThe search will strip these symbols and ignore them.","ref":"filtering.html#quoted-searches"},{"type":"extras","title":"Suggestion Caching - Filtering","doc":"Suggestions are lazily fetched and cached for subsequent lookups. By default, suggestion queries\nare limited to the most recent 10k jobs to minimize database load. You can change the limit with\nthe `c:Oban.Web.Resolver.hint_query_limit/1` callback in a custom resolver.\n\nCached items are purged after a fixed 5 minute period, which isn't configurable.","ref":"filtering.html#suggestion-caching"},{"type":"extras","title":"Metrics","doc":"# Metrics\n\nWeb's realtime updates, counts, and charts are powered by the [Oban.Met][met] package. `Met`\nprovides core monitoring and introspection functionality from a single automatically managed\nsupervisor.\n\n* Telemetry powered metric tracking and aggregation with compaction\n* Periodic queue checking and reporting (replaces the `Gossip` plugin)\n* Periodic counting and reporting with backoff (replaces `Stats` plugin)\n* Leader backed distributed metric sharing with handoff between nodes\n\n[met]: https://github.com/oban-bg/oban_met","ref":"metrics.html"},{"type":"extras","title":"Usage in Worker Only Nodes - Metrics","doc":"To receive metrics from non-web nodes in a system with separate \"web\" and \"worker\" applications\nyou must explicitly include `oban_met` as a dependency for \"workers\".\n\n```elixir\n{:oban_met, \"~> 1.0\"},\n```","ref":"metrics.html#usage-in-worker-only-nodes"},{"type":"extras","title":"Auto Start - Metrics","doc":"Supervised `Met` instances start automatically along with Oban instances unless Oban is in\ntesting mode. You can disable auto-starting globally with application configuration:\n\n```elixir\nconfig :oban_met, auto_start: false\n```\n\nHowever, note that a running `Met` instance is required for the Web dashboard to load and without\none the dashboard won't function.","ref":"metrics.html#auto-start"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.0"}}